
/*
 *    CSFrame.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
package org.nzdl.gsdl.SimpleGraphicalClient;


import javax.swing.JMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JList;
import javax.swing.JPopupMenu;
import javax.swing.BoxLayout;
import javax.swing.JMenu;
import javax.swing.AbstractAction;
import javax.swing.SwingUtilities;
import javax.swing.KeyStroke;

import java.awt.*;
import java.awt.event.*;
import java.util.Observer;
import java.util.Observable;
/**
 * Class 
 *
 * A class to 
 * 
 * @author Dave Nichols (daven@cs.waikato.ac.nz)
 * @author stuart yeates (say1@cs.waikato.ac.nz)
 * @version $Revision: 2260 $
 * @see org.nzdl.gsdl.service.SimpleGraphicalClient.SimpleGraphicalClient
 */

public class CSFrame extends JFrame implements Observer, Constants

{

CSModel csModel;
SearchPanel searchPanel;
JPanel otherPanel;


  // Constructor
  public CSFrame(String title, CSModel newCSModel)
  {
    setTitle(title);                              // Set the window title
	
	csModel = newCSModel;
	Container content = getContentPane();

    setJMenuBar(menuBar);                         // Add the menu bar to the window

    JMenu fileMenu = new JMenu("File");           // Create File menu
    JMenu editMenu = new JMenu("Edit");    	   // Create Edit menu

    // File Menu

    //fileMenu.addSeparator();
    fileMenu.add( quitAction = new FileAction("Poll Server", POLL_ID, this)).setAccelerator(KeyStroke.getKeyStroke('P', Event.CTRL_MASK));
    fileMenu.add( quitAction = new FileAction("Collect Garbage", GARBAGE_ID, this)).setAccelerator(KeyStroke.getKeyStroke('G', Event.CTRL_MASK));
    fileMenu.add( quitAction = new FileAction("Quit", QUIT_ID, this)).setAccelerator(KeyStroke.getKeyStroke('Q', Event.CTRL_MASK));

    // Edit Menu

    editMenu.add( prefAction = new EditAction("Preferences", PREF_ID, this));
    editMenu.add( prefAction = new EditAction("View Change Log", LOG_ID, this));

	
    menuBar.add(fileMenu);                        // Add the file menu
    menuBar.add(editMenu);                        // Add the edit menu
	
	
	searchPanel = new SearchPanel(csModel, this);
		
	JPanel centerPanel = new JPanel();
	QueryHistoryPanel queryHistoryPanel = new QueryHistoryPanel(csModel);
	centerPanel.add(queryHistoryPanel);

	content.setLayout(new BoxLayout(content, BoxLayout.X_AXIS));	
	content.add(searchPanel);
	content.add(centerPanel);
	
	addWindowListener(new WindowHandler());
		
	pack();
	searchPanel.searchTextField.requestFocus();
  } //end constructor


public void update(Observable observedThing, Object o )
	{
	System.out.println("Updating...");
	//jlist.setListData( ((CSModel)observedThing).getModel() );
	}



  // action objects

  abstract class CSAction extends AbstractAction {
      int menuItemID;
      CSFrame frame;
  
  		  CSAction( String name, int newMenuItemID, CSFrame f)
  		  {
  		   	super(name);
			menuItemID = newMenuItemID;
			frame = f;
  		   }
		   
		   public abstract void actionPerformed(ActionEvent e);
		   
  } //end CSAction
  
  class FileAction extends CSAction
  {
        FileAction(String name, int newMenuItemID, CSFrame frame)
        {
             super(name,newMenuItemID, frame );
         }

         public void actionPerformed(ActionEvent e)
         {
		 switch (menuItemID) {
		 		 case GARBAGE_ID:
		   System.err.println("forcing gc...");
		   System.gc() ;
		   System.runFinalization();
		   System.err.println("forcing gc...");
		   System.gc() ;
		   System.runFinalization();
		   System.err.println("forced gc");
		   break; 
				case POLL_ID:
					 System.out.println("poll chosen from menu");
					 break; 
				case QUIT_ID:
					 System.out.println("quit chosen from menu");
					 dispose();       // free resources
					 System.exit(0);  // exit program
					 break; 
				default:
					 System.out.println("unimplemented option chosen from FileAction");
					 break; 
		  } // end switch
         }//end actionPerformed
  } // end FileAction

  class EditAction extends CSAction 
  {
        EditAction(String name, int newMenuItemID, CSFrame frame)
        {
             super(name,newMenuItemID, frame );
         }

         public void actionPerformed(ActionEvent e)
         {
		 switch (menuItemID) {
		 		case PREF_ID:
					 PreferencesDialog prefDialog = new PreferencesDialog(frame, "Preferences", false); 
					 break;
		 		case LOG_ID:
					 ChangeLogDialog logDialog = new ChangeLogDialog(frame, "ChangeLog", false); 
					 break;
		 		default:
					 System.out.println("unimplemented option chosen from EditAction");
					 break; 
		 } // end switch
         } //end actionPerformed
  } // end EditAction
  
  
class WindowHandler extends WindowAdapter
{

   public void windowOpened(WindowEvent e) {

    SwingUtilities.invokeLater(new Runnable() {
       public void run() {
           searchPanel.searchTextField.requestFocus();
		   searchPanel.searchTextField.setCaretPosition(
		      searchPanel.searchTextField.getText().length() );
		   searchPanel.searchTextField.selectAll();
       }
    });
	} //end windowOpened



	public void windowClosing(WindowEvent e)
	{
		dispose();       // free resources
		System.exit(0);  // exit program
	}
}// end WindowHandler


class MouseHandler extends MouseAdapter
{
private Frame frame;

 		public MouseHandler( CSFrame f )
 		{
		   this.frame = f; 
		}

     public void mouseClicked(MouseEvent e) {
	 		//System.out.println("mouse event" );
         if (e.getClickCount() == 2) {
		 	 System.out.println("double click - mouse event" );
             int index = jlist.locationToIndex(e.getPoint());
			 System.out.println("index is " + index );
			 //psObject = (PeopleSpaceObject) jlist.getModel().getElementAt(index);
			 } //end if
     } //end mouseClicked
		  
		  public void mousePressed(MouseEvent e) {
          		 maybeShowPopup(e);
          }

    	  public void mouseReleased(MouseEvent e) {
          		 maybeShowPopup(e);
    	  }

    	  private void maybeShowPopup(MouseEvent e) {
          		  if (e.isPopupTrigger()) {
            	  	   contextPopup.show(e.getComponent(),
                       e.getX(), e.getY());
          		  } //end if	  
  		  } //end maybeShowPopup
  
 } //end class MouseHandler

  // private declarations
  private JMenuBar menuBar = new JMenuBar();      // Window menu bar
  private JMenuItem  quitItem;
  
 private JList jlist, peopleList;
 private JPopupMenu contextPopup;

  // FileActions
  private FileAction newSpaceAction, newGroupAction, newPersonAction, quitAction;
  
  // EditActions
  
  private EditAction prefAction;

  //Dialog

  private PreferencesDialog newPreferencesDialog;
  
} //end CSFrame
