/*
 *    NzdlWrapper.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//the package we're in
package org.nzdl.gsdl.service;

import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.Date;
import java.io.Writer;
import java.io.FileWriter;


/**
 *  A Wrapper around an NzdlService object that adds logging functionality. 
 *
 * @author Brett Sheeran (bsheeran@xtra.co.nz) (comments)
 * @author Stuart Yeates  (say1@cs.waikato.ac.nz)
 * @version $Revision: 2220 $
 */
public class NzdlLogWrapper
  extends NzdlWrapper {

  Writer logFile = null;
  String tag = "NzdlLogWrapper: ";
  boolean calculateDiff = true;
  boolean logBefore = false;
  boolean logAfter = false;
  Date whenCallStarted = null;
   

  /** 
   * The bean constructor ...
   */
  public NzdlLogWrapper() {
  }
  /** 
   * The normal constructor
   */
  public NzdlLogWrapper(NzdlService nzdl, String logFilename, 
			String tag) {
    super(nzdl);
    this.tag = tag;
    try {
      Date date = new Date();
      // the true is append ...
      logFile = new FileWriter(logFilename, true);
      logFile.write(tag + "Log started at:" + date + " == " + 
		    date.getTime() + "\n");
      logFile.write(tag + "timestamps in milliseconds since January 1, 1970, 00:00:00 GMT \n");
      logFile.flush();
    } catch (Exception exception) {
      System.err.println("unable to create log file:" + exception);
    }    
  }

  protected void logBefore(String method) {
    if (whenCallStarted != null)
      throw new Error ("synchronisation required ! (whenCallStarted != null)");
    whenCallStarted = new Date();
    if (logBefore)
      try {
	logFile.write(this.tag + "before " + method + "() " + 
		      whenCallStarted.getTime() + "\n");
	logFile.flush();
      } catch (Exception exception) {
	System.err.println("unable to write log file:" + exception);
      } 
  }
   
  protected void logAfter(String method) {
    if (whenCallStarted == null)
      throw new Error ("synchronisation required ! (whenCallStarted == null)");
    Date now = new Date();
    try {
      if (logAfter)
	logFile.write(this.tag + "after " + method + "() " + 
		      whenCallStarted.getTime() + "\n");
      if (calculateDiff)
	logFile.write(this.tag + "time for  " + method + "() " + 
		      (now.getTime() -whenCallStarted.getTime()) + "ms\n");
      logFile.flush();
    } catch (Exception exception) {
      System.err.println("unable to write log file:" + exception);
    } 
    whenCallStarted = null;
  }
   
  /**
   * Allow the client to configure a CORBA server.
   * @param key the parameter to be set.
   * @param values what that parameter will be set to.
   */
  public void configure( String _key, Set _values ) {
    logBefore("configure");
    super.configure(_key, _values);
    logAfter("configure");
  }

  /**
   * Obtain the set of collection names for a Greenstone library.
   * @return A set of collection names in string format.
   */
  public Set getCollectionSet( ) {
    logBefore("getCollectionSet");
    Set result = super.getCollectionSet();
    logAfter("getCollectionSet");
    return result;
  }

  /**
   * Check if ORB knows of this collection.
   * @param name the name of a collecion to be searched for.
   * @return boolean <TT>true</TT> if collection is found, else <TT>false</TT>.
   */
  public boolean hasCollection( String _name ) {
    logBefore("hasCollection");
    boolean result = super.hasCollection(_name);
    logAfter("hasCollection");
    return result;
  }


  /**
   * Check if ORB can communicate with of this collection.
   * @param name the name of a collecion to be pinged.
   * @return boolean <TT>true</TT> if collection returned ping, else
   * <TT>false</TT>.
   */
  public boolean pingCollection( String _name ) {
    logBefore("pingCollection");
    boolean result = super.pingCollection(_name);
    logAfter("pingCollection");
    return result;
  }

  /**
   * Extract information on this collection into a
   * {@link NzdlCollectionInfo NzdlCollectionInfo} object
   * @param name the collection to be retrieved.
   * @return an instance of
   * {@link NzdlCollectionInfo NzdlCollectionInfo} containing data on a
   * collection
   */
  public NzdlCollectionInfo getCollectionInfo( String _name ) {
    logBefore(" getCollectionInfo");
    NzdlCollectionInfo  result = super.getCollectionInfo(_name);
    logAfter(" getCollectionInfo");
    return result;
  }

  /**
   * Returns the set of filters for the service object.
   * @param name the collection name
   * @return a set of strings that name the filters, for example
   * BrowseFilter, QueryFilter and NullFilter.
   */
  public Set getFilterSet( String _name ) {
    logBefore("getFilterSet");
    Set result = super.getFilterSet(_name);
    logAfter("getFilterSet");
    return result;
  }

  /**
   * @param name the collection name
   * @return the Set of (name, value) value pairs for all metadata 
   * associated with a collection  
   */
  public Set getMetaTagSet( String _name ) {
    logBefore("getMetaTagSet");
    Set result = super.getMetaTagSet(_name);
    logAfter("getMetaTagSet");
    return result;
  }

  /**
   * Returns the Greenstone Markup Language (GML) text for a particular
   * document from a collection.
   * @param name the name of the collection.
   * @param docID the document identity string
   * @return the GML document in the form of a string.
   */
  public String getDocument( String _name, String _docID ) {
    logBefore("getDocument");
    String result = super.getDocument(_name, _docID);
    logAfter("getDocument");
    return result;
  }

  /**
   * Returns the set of value(s) for a metatag from a particular document.
   * @param name collection name
   * @param docID the document identifier.
   * @param metaTag the metatag name such as: Title, Author, Date and Images.
   * @return The set of value(s) for the requested metatag and document. In
   * cases such as Title, Author, Date there will probably be only one value
   * per document.
   */
  public Set getMetaData( String _name, String _docID, String _metaTag ) {
    logBefore("getMetaData");
    Set result = super.getMetaData( _name, _docID, _metaTag );
    logAfter("getMetaData");
    return result;
  }

  /**
   * Returns a map of value(s) for a metatag from a list of documents.
   * @param name collection name
   * @param docIDs a list of document identifier strings.
   * @param metaTag the metatag name such as: Title, Author, Date and Images.
   * @return The map of value(s) for the requested metatag and document. In
   * cases such as Title, Author, Date there will probably be only one value
   * per document.
   */
  public Map getMetaData( String _name, List _docIDs, String _metaTag ) {
    logBefore("getMetaData");
    Map result = super.getMetaData( _name, _docIDs, _metaTag );
    logAfter("getMetaData");
    return result;
  }

  /**
   * Services a {@link NzdlRequest NzdlRequest} to a collection and places
   * the results into a {@link NzdlResponse NzdlResponse} object. <br>
   * Pre:   Created instances of NzdlRequest and NzdlResponse. <br>
   * Post: Placed the result data into NzdlResponse object.
   * @param name the collection name.
   * @param request an instance of NzdlRequest with query data to be serviced
   * @param response an instance of NzdlResponse. where the results of
   * sevciving the query will be placed
   */
  public void service( String _name, NzdlRequest _request, 
		       NzdlResponse _response ) {    
    logBefore("service");
    super.service(_name,_request,_response);
    logAfter("service");
  }
  
}
